/* Copyright (C) 1999 Jeff Heflin - University of Maryland
 *	
 * This software is provided under the terms of the GNU Lesser General Public
 * License Version 2.1, February 1999. See the header in HTML_Tokenizer.java
 * and the file License.html for details.
 */

package HTML;

import java.net.*;
import java.io.*;

/** This class is a URL that allows the user to set up a proxy server
  for retrieving URLs. If this server is set and it is different than
  the host of the requested URL, then the request to fetch the URL
  will be made to the proxy server. Otherwise, the fetch will be made
  directly. This is mainly intended so that an applet can use a proxy 
  server on the remote host to load URLs from other servers. */
public class AltProxyUrl
{

  private static final boolean LOG_GETS = false;

  private static String webProxyHost;
  private static int    webProxyPort;

  private URL  url;

  public AltProxyUrl(String protocol, String host, int port, String file)
       throws MalformedURLException
  {
    url = new URL(protocol, host, port, file);
  }

  public AltProxyUrl(String protocol, String host, String file)
       throws MalformedURLException
  {
    url = new URL(protocol, host, file);
  }

  public AltProxyUrl(String spec)
       throws MalformedURLException
  {
    url = new URL(spec);
  }

  public AltProxyUrl(URL context, String spec)
       throws MalformedURLException
  {
    url = new URL(context, spec);
  }

  public static void setWebProxy(String proxyHost, int proxyPort)
  {
    // if host is empty string, treat it as if there is no host
    if ("".equals(proxyHost)) {
      proxyHost = null;
    }
    webProxyHost = proxyHost;
    webProxyPort = proxyPort;    
    // System.out.println("Host=" + webProxyHost + ", Port=" + webProxyPort);
  }


  public InputStream openStream() throws IOException
  {
    InputStream inputStream = null;

    if (webProxyHost == null || webProxyHost.equals(url.getHost())) {
      // if no proxy set or desired host is applet server, open directly
      if (LOG_GETS == true)
	System.out.println("Opening URL stream " + url.toString() + " directly...");
      inputStream = url.openStream();
    }
    else {
      Socket          socket;
      PrintStream     outputStream = null;
	  
      // use proxy server to get web page
      if (LOG_GETS == true)
	System.out.println("Opening URL stream " + url.toString() + " using proxy server...");
      try {
	socket = new Socket(webProxyHost, webProxyPort);
	outputStream = new PrintStream(new BufferedOutputStream(socket.getOutputStream()));
	inputStream = socket.getInputStream();
	
	outputStream.println("GET " + url.toExternalForm());
	outputStream.flush();
	
      } catch (IOException ex) {
	System.out.println("Error connecting to proxy server " + 
			   webProxyHost + ":" + webProxyPort);
	// try to open URL regularly
	inputStream = url.openStream();
      }
    }
    return inputStream;
  }
}

