/* Copyright (C) 1999 Jeff Heflin - University of Maryland
 *	
 * This software is provided under the terms of the GNU Lesser General Public
 * License Version 2.1, February 1999. See the header in HTML_Tokenizer.java
 * and the file License.html for details.
 */

package HTML;

import java.lang.System;
import java.net.*;
import java.io.*;
import java.util.*;


public class Exclusions extends Object {

 Hashtable exTable;

  public Exclusions() {

	 exTable = new Hashtable();
  }

  String makeKey(String server, int port) {
    /* Make the hashtable key for a server's exclusion set. Since the same
       address could have many host names, we use the internet address as
       our key */

    InetAddress address = null;

    try {
      address = InetAddress.getByName(server);
    } catch (UnknownHostException ex) {
      System.out.println("Unknown host: " + server);
    }
    return address.getHostAddress();
    // return address;
  }

  void getExclusions(String server, int port) {
    /* read the robots.txt file for a server */
    /* Note: this function currently stores the list of excluded files as
       an unsorted Vector. Thus, duplicates may appear and searches can be
       slow. */
    
    Vector exFiles;
    String key;
    int tokeType;
    String nextLine;
    
    exFiles = new Vector();

    /* read and parse robot file */
    URL url = null;
    try { 
      url = new URL("http", server, port, "/robots.txt");
    } catch (MalformedURLException e) {
      // something weird is happening, so don't trust it
      System.out.println("Malformed URL for robots.txt");
    }
    System.out.println("Checking for robot exclusions on " + url.getHost());

    try {
      DataInputStream urlStream = new DataInputStream(url.openStream());

      while ((nextLine = urlStream.readLine()) != null) {
	/* assume that this robots.txt refers to us and 
	   search for "Disallow:" commands. */
	if (nextLine.toLowerCase().startsWith("disallow:")) {
	  StringTokenizer st = new StringTokenizer(nextLine.substring(9));
	  if (st.hasMoreTokens()) {
	    String exclude = st.nextToken();
	    // System.out.println("Excluded: " + exclude);
	    exFiles.addElement(exclude);
	  }
	}
      }
    } catch (IOException e) {
      // if there is no robots.txt file, it is OK to search
      System.out.println("No robots.txt");
    }
    /* insert entry for this host into the hashtable */
    key = makeKey(server, port);
    exTable.put(key, exFiles);
  }

  public boolean isExcluded(URL url) {

    String server;
    int port;
    String file;
    String key;
    int i;
    Vector exFiles;
    String ex;

    server = url.getHost();
    port = url.getPort();
    file = url.getFile();

    key = makeKey(server, port);
    if (exTable.containsKey(key) == false) {
      /* create entry for a new server */
      getExclusions(server, port);
    }
    exFiles = (Vector)exTable.get(key);
    if (exFiles == null)
      System.out.println("Didn't get a file  list for server " + server);
    for (i=0; i < exFiles.size(); i++) {
      ex = (String)exFiles.elementAt(i);
      if (file.startsWith(ex))
	return true;
    }
    return false;
  }

  public String toString() {

    return exTable.toString();
  }
}
