/* Copyright (C) 1999 Jeff Heflin - University of Maryland
 *	
 * This software is provided under the terms of the GNU Lesser General Public
 * License Version 2.1, February 1999. See the header in SHOE_Doc.java
 * and the file License.html for details.
 */

/* Change History:
   11/01/00, JDH: Made all fields private. Added accessor methods for them.
*/

package SHOE;

import java.util.*;
import java.io.*;
import HTML.*;

/** A class for representing the definition of a SHOE constant. A constant
  is essentially an instance declared in the ontology to promote
  the interoperability of shared concepts. */
public class Def_Constant extends SHOE_Obj {
	
  private String name;
  private String category_name;
  private Unique_Elt categoryId;

  /** Create a new constant given its name and the prefixed name
    of the category of which it is a member. */
  public Def_Constant(String name, String category_name) {

    this.name = name;
    this.category_name = category_name;
  }

  /** Create a new constant given its SGML tag representation. */
  public Def_Constant(SGML_Tag sgmltag) {
 
     name = sgmltag.getAttribute("NAME");
     category_name = sgmltag.getAttribute("CATEGORY");
  }

  /** Return the prefixed name of the category that this constant is
    classified under. */
  public String getCategoryName() {

    return category_name;
  }

  /** Return a string that identifies the constant within the context
    of the ontology. */
  public String getKey() {
  
    return name;
  }

  /** Return the name of the constant. */
  public String getName() {

    return name;
  }

  /** Return an ontology independent identifier for the category
    of which this constant is a member. */
  public Unique_Elt getCategoryId() {

    return categoryId;
  }

  public Vector readSubTags(HTML_Tokenizer intokens) {

    Vector errStrings = null;

    errStrings = new Vector();
    return errStrings;
  }

  /** Check the constant object for errors and update internal structures. */
  public Vector validate(SHOE_Obj parent, OntManager ontman) {

    Vector errStrings;

    errStrings = new Vector();
    try {
      categoryId = ontman.resolveName(category_name, parent);
    } catch (SHOEException ex) {
      errStrings.addElement(ex.getMessage());
    }
    return errStrings;
  }

  public void printSGML(PrintStream out) {
    
    out.println(this);
  }

  /** Returns the string representation of the tag. Any contents
   or end tags are not included. */
  public String toString() {
  
    String str;
    
    str = "<DEF-CONSTANT NAME=\"" + name + "\"";
    if (category_name != null)
      str = str + " CATEGORY=\"" + category_name + "\"";
    str = str + ">";
    return str;
  }
}

