/* Copyright (C) 1999 Jeff Heflin - University of Maryland
 *	
 * This software is provided under the terms of the GNU Lesser General Public
 * License Version 2.1, February 1999. See the header in SHOE_Doc.java
 * and the file License.html for details.
 */

/*
  Change History:
    11/01/00, JDH: Made from, to and elmt_id fields private.
                   Added accessor methods
*/

package SHOE;

import java.util.*;
import java.io.*;
import HTML.*;

public class Def_Rename extends SHOE_Obj {
	
  private String from = null;      // source object's name
  private String to = null;        // new local name for object
  private Unique_Elt elmt_id;              // unique id of source object

  /** Create a new rename object given the prefixed name of the object
    to be renamed and the new local name. */
  public Def_Rename(String from, String to)  {

    this.from = from;
    this.to = to;
  }

  /** Create a new rename object given its SGML tag representation. */
  public Def_Rename(SGML_Tag sgmltag) {
 
     from = sgmltag.getAttribute("FROM");
     to = sgmltag.getAttribute("TO");
  }

  /** Returns the prefixed name of the source object (the object that is
    renamed. */
  public String getFrom() {
    
    return from;
  }

  /** Returns the new local alias of the renamed object. */
  public String getTo() {

    return to;
  }

  /** Returns a unique identifier for the rename. */
  public String getKey() {
  
    if (to == null)
      return "missingkey";
    else
      return to;
  }

  /** Returns an ontology independent unique identifier for the
    source object. */
  public Unique_Elt getFromId() {

    return elmt_id;
  }

  public Vector readSubTags(HTML_Tokenizer intokens) {

    Vector errStrings = null;

    errStrings = new Vector();
    return errStrings;
  }

  /** Check the Def_Rename for errors and update internal structures.
    Stores a pointer to the objects referenced by the source name.
    */
  public Vector validate(SHOE_Obj parent, OntManager ontman) {

    Vector errStrings;
    SHOEException perr;

    // System.out.println("Validating " + toString());
    errStrings = new Vector();

    if (from != null) {     // no need to resolve if field is missing
      try {
	elmt_id = ontman.resolveName(from, parent);
      } catch (SHOEException ex) {
	errStrings.addElement(ex.getMessage());
      }
    }
    return errStrings;
  }

  public void printSGML(PrintStream out) {
  
    out.println(this);
  }

  /** Returns the string representation of the tag. Any contents
   or end tags are not included. */
  public String toString() {
  
    String str;
    
    str = "<DEF-RENAME FROM=\"" + from + "\"";
    str = str + " TO=\"" + to + "\"";
    str = str + ">";
    return str;
  }
}
