/* Copyright (C) 1999 Jeff Heflin - University of Maryland
 *	
 * This software is provided under the terms of the GNU Lesser General Public
 * License Version 2.1, February 1999. See the header in SHOE_Doc.java
 * and the file License.html for details.
 */

/*
  Change History:
    01/10/01, JDH: Changed short_para to shortPhrase
    11/01/00, JDH: Changed all fields to private. Added accessor methods.
*/

package SHOE;

import java.util.*;
import java.io.*;
import HTML.*;

/** A class for representing a SHOE type definition. Note that SHOE
  does not provide the expressivity to define complex types, the
  type mechanism is merely a place holder so that the custom applications
  can know when special typing mechanisms might apply. */
public class Def_Type extends SHOE_Obj {
	
  private String name;
  private String description;
  private String shortPhrase;
  
  /** Create a new type given its name, description, and a short phrase. */
  public Def_Type(String name, String description, String shortPhrase) {

    this.name = name;
    this.description = description;
    this.shortPhrase = shortPhrase;
  }

  /** Create a new type given its SGML tag representation. */
  public Def_Type(SGML_Tag sgmltag) {
 
     name = sgmltag.getAttribute("NAME");
     description = sgmltag.getAttribute("DESCRIPTION");
     shortPhrase = sgmltag.getAttribute("SHORT");
  }

  /** Return a description of the type. */
  public String getDescription() {

    return description;
  }

  /** Return a string that identifies the type within the context
    of the ontology. */
  public String getKey() {
  
    return name;
  }

  /** Return the name of the type. */
  public String getName() {

    return name;
  }

  /** Returns a short phrase that describes the type. This is
    the SHORT field in the DEF-TYPE tag. */
  public String getPhrase() {

    return shortPhrase;
  }

  public Vector readSubTags(HTML_Tokenizer intokens) {

    Vector errStrings = null;

    errStrings = new Vector();    
    return errStrings;
  }


  public void printSGML(PrintStream out) {
    
    out.println(this);
  }


  /** Returns the string representation of the tag. Any contents
   or end tags are not included. */
  public String toString() {
  
    String str;
    
    str = "<DEF-TYPE NAME=\"" + name + "\"";
    if (description != null)
      str = str + " DESCRIPTION=\"" + description + "\"";
    if (shortPhrase != null)
      str = str + " SHORT=\"" + shortPhrase + "\"";
    str = str + ">";
    return str;
  }

}

