/* Copyright (C) 1999 Jeff Heflin - University of Maryland
 *	
 * This software is provided under the terms of the GNU Lesser General Public
 * License Version 2.1, February 1999. See the header in SHOE_Doc.java
 * and the file License.html for details.
 */

package SHOE;

import java.util.*;
import java.io.*;
import HTML.*;

/** A class representing an argument of a relationship. */
public class RelArg extends SHOE_Obj {

  private int pos = 0;                   // position number of argument
  private String strval;

  /** The id for an ontology object refered to by this argument */
  private Unique_Elt ontRefId = null;   

  private byte usage;                    // variable or value?
  private boolean textPos = false;       // use "TO/FROM" string for position?

  //  private int intval;                        // not used
  //  private boolean isDefault = false;

 
  /** Create a new relation argument with the given position and value. */
  public RelArg(int pos, String value) {
		
    this.pos = pos;	
    this.strval = value;
    this.usage = U_VAL;
  }

  /*
  public RelArg(int pos, int value) {

    this.pos = pos;
    this.intval = value;
    this.usage = U_VAL;
  }
  */

  /** Create a new relation argument from its SGML tag representation. */
  public RelArg(SGML_Tag sgmltag) {

    String posstr;
    String utext;
    String str;

    posstr = sgmltag.getAttribute("POS");
    if (posstr != null) {
      if (posstr.equalsIgnoreCase("FROM")) {
        pos = 1;
	textPos = true;
      }
      else if (posstr.equalsIgnoreCase("TO")) {
        pos = 2;
	textPos = true;
      }
      else {
	try {
	  pos = Integer.parseInt(posstr);
	} catch (NumberFormatException ex) {
	  pos = 0;
	}
      }
    }

    strval = sgmltag.getAttribute("VALUE");
    usage = U_VAL;
    utext = sgmltag.getAttribute("USAGE");
    if (utext != null) {                             // form USAGE="VAR"
      if (utext.equalsIgnoreCase("VAR"))
	usage = U_VAR;
    }
    else {                                    // check for simple VAR
      str = sgmltag.findValName("VAR");
      if (str != null && str.equals(""))
	usage = U_VAR;
    }
  }
  
  public String getKey() {
  
    return Integer.toString(pos);

  }

  /** Retrun the value of the argument. */
  public String getValue() {
    
    return strval;
  }

  /** Return the argument number of the argument. */
  public int getPosition() {

    return pos;
  }

  /** If the argument value is a reference to an ontology element, return
    the unique element id of this element. */
  public Unique_Elt getOntReference() {

    return ontRefId;
  }


  /** Returns true if the argument is a variable. */
  public boolean isVariable() {

    if (usage == U_VAR)
      return true;
    else
      return false;
  }

  public Vector readSubTags(HTML_Tokenizer intokens) {

    Vector errStrings = null;

    errStrings = new Vector();
    return errStrings;
  }

  /** Checks the argument for errors and updates internal information.
   */
  public Vector validate(SHOE_Obj parent, OntManager ontman) {

    Vector errStrings;
    String name;

    errStrings = new Vector();

    if (strval != null) {
      /* Replace ME arguments with instance key */
      if (strval.equalsIgnoreCase("ME")) {
	if (parent instanceof Instance)
	  strval = parent.getKey();
	else                                                   // don't change
	  System.out.println("Used ME in a Def-Inference");
      }
      
      /* validate constants */
      if (strval.startsWith("!")) {
	name = strval.substring(1);
	try {
	  ontRefId = ontman.resolveName(name, parent);
	} catch (SHOEException ex) {
	  errStrings.addElement(ex.getMessage());
	  return errStrings;
	}
      }
    }
    return errStrings;
  }

  /** Sets the value of the argument. */
  public void setValue(String val) {

    strval = val;
  }

  /** Sets whether text strings should be used as the values of the
    POS attribute when printing the tag. If true, then "FROM" and "TO"
    are used in place of the integer values "1" and "2". */
  protected void setPosAsText(boolean textPos) {

    this.textPos = textPos;
  }

  public void printSGML(PrintStream out) {
  
    out.print(this);
  }

  /** Returns the string representation of the tag. Any contents
   or end tags are not included. */
  public String toString() {
  
    StringBuffer str;
    
    str = new StringBuffer("<ARG POS=");
    if (textPos == false)
      str.append("\"" + pos + "\"");
    else {                       // use textual version of argument
      if (pos == 1)
	str.append("\"FROM\"");
      else if (pos == 2)
	str.append("\"TO\"");
    }
    if (usage == U_VAR)
      str.append(" USAGE=\"VAR\"");
    str.append(" VALUE=\"" + strval + "\"");
    str.append(">");
    return str.toString();
  }
  
}
