/* Copyright (C) 1999 Jeff Heflin - University of Maryland
 *	
 * This software is provided under the terms of the GNU Lesser General Public
 * License Version 2.1, February 1999. See the header in SHOE_Doc.java
 * and the file License.html for details.
 */

/* Change History:
   11/01/00, JDH: Added NOT_RELATION and NOT_CATEGORY errors
*/
 
package SHOE;

import java.util.*;

public class SHOEException extends Exception {

  public static final int INCOMPATIBLE_VERSION = 0;
  public static final int NO_CLOSING_TAG = 1;
  public static final int UNEXPECTED_TAG = 2;
  public static final int DUPLICATE_PREFIX = 3;
  public static final int NO_ELEMENT_DEF = 4;
  public static final int MISSING_PREFIX = 5;
  public static final int MISSING_ONTOLOGY = 6;
  public static final int PREFIX_NOT_DEFINED = 7;
  public static final int DUPLICATE_NAME = 8;
  public static final int NO_ONTOLOGY_FILE = 9;
  public static final int INVALID_TYPE = 10;
  public static final int MISSING_FIELD = 11;
  public static final int INVALID_ARG_POS = 12;
  public static final int MISSING_ARG_VALUE = 13;
  public static final int NOT_RELATION = 14;
  public static final int NOT_CATEGORY = 15;

  public static final String[] Messages =
  {"Not compatible with document SHOE Version <1>",
   "Tag <1> is missing closing tag <2>",
   "Unexpected tag <1> inside declaration",
   "Prefix '<1>' duplicates the prefix of another ontology",
   "Element <1> not found in ontology <2>",
   "Declaration name <1> needs an ontology prefix",
   "Could not find ontology corresponding to prefix <1>",
   "Prefix '<1>' does not match a USE-ONTOLOGY statement",
   "Tag <1> duplicates the name of another element in the ontology",
   "Cannot find ontology file specified in Use-Ontology <1>",
   "<1> is not a valid argument type",
   "Tag <1> is missing required field <2>",
   "Relation <1> contains an invalid value for the POS field of an argument",
   "Relation <1> has no value in argument <2>",
   "<1> is not a relation in ontology <2>",
   "<1> is not a category in ontology <2>"};

  private String message;

  /** Given a vector of current error strings, an error header string, and
     a vector of errors to be inserted under the header, adds the header
     and new error strings to the current vector */
  public static void addSubErrors(Vector errStrings, String header,
			     Vector subErrs) {

    Enumeration errEnum;
    String errStr;

    if (subErrs.size() > 0) {
      errEnum = subErrs.elements();	  
      errStrings.addElement(header);

      /* this while loop is very slow. Is there a simple way to do this 
	 more efficiently? */
      while (errEnum.hasMoreElements()) {
	errStr = "  " + (String)errEnum.nextElement();
	errStrings.addElement(errStr);
      }
    }
  }

  public SHOEException(int err_id) {

    message = Messages[err_id];
  }

  public SHOEException(int err_id, String arg1) {

    String msg;
    int a1, a2;

    msg = Messages[err_id];
    a1 = msg.indexOf("<1>");
    a2 = msg.indexOf("<2>");
    if (a1 < 0 || a2 >= 0) {
      System.out.println("Code error: Created Exception " + msg +
			 " with one argument " + arg1);
      return;
    }
    message = msg.substring(0, a1) + arg1 + msg.substring(a1 + 3);
  }

  public SHOEException(int err_id, String arg1, String arg2) {

    String msg;
    int a1, a2;

    msg = Messages[err_id];
    a1 = msg.indexOf("<1>");
    a2 = msg.indexOf("<2>");
    if (a1 < 0 || a2 < 0) {
      System.out.println("Code error: Created Exception " + msg +
			 " with two arguments " + arg1 + " and " + arg2);
      return;
    }
    message = msg.substring(0, a1) + arg1 + msg.substring(a1 + 3, a2) +
      arg2 + msg.substring(a2 + 3);
  }

  /* Overrides java.lang.Throwable */
  public String getMessage() {

    return "Error: " + message;
  }
}
