/* Copyright (C) 1999 Jeff Heflin - University of Maryland
 *	
 * This software is provided under the terms of the GNU Lesser General Public
 * License Version 2.1, February 1999. See the header in SHOE_Doc.java
 * and the file License.html for details.
 */

package SHOE;

import java.lang.System;
import java.io.*;
import java.util.*;
import SHOE.*;

/** A test harness for various functionality of the SHOE library. */
public class SHOE_Test {

  public static void main(String args[]) throws IOException {
  
    int type;
    OntManager ontman = new OntManager();
    Unique_Elt elmt_id;
    Instance inst;
    String filename = "cs.html";
    Enumeration enum;

    /*
      // File parsing test
    DataInputStream kbd = new DataInputStream(System.in);
    System.out.println("Enter the file to be read:");
    filename = kbd.readLine();
    FileInputStream inStream = new FileInputStream(filename);
    SHOE_Doc doc = new SHOE_Doc();

    System.out.println("Parsing...");
    doc.parse(inStream);
    System.out.println("Validating ...");
    doc.validate(ontman);
    System.out.println("Errors:");
    System.out.println(doc.getParseErrors());
    */

    // OntMan.isSubcategoryOf() test
    Ontology ont;
    int i;
    Unique_Elt childId, parId;
    Date startTime, endTime;
    long duration;
    boolean answer;

    String[][] ids = {{"cs-dept-ontology", "1.0", "Worker",
		       "cs-dept-ontology", "1.0", "Person"},
		      {"cs-dept-ontology", "1.0", "VisitingProfessor",
		       "cs-dept-ontology", "1.0", "Person"},
		      {"cs-dept-ontology", "1.0", "Organization",
		       "cs-dept-ontology", "1.0", "Person"},
		      {"cs-dept-ontology", "1.1", "UnofficialPublication",
		       "document-ont", "1.0", "Publication"},
		      {"cs-dept-ontology", "1.1", "UnofficialPublication",
		       "general-ont", "1.0", "Agent"},
		      {"university-ont", "1.0", "VisitingProfessor",
		       "general-ont", "1.0", "Agent"},
		      {"university-ont", "1.0", "Chair",
		       "general-ont", "1.0", "Agent"},
		      {"university-ont", "1.0", "Chair",
		       "general-ont", "1.0", "Organization"},
    };

    ont = ontman.getOntology("cs-dept-ontology", "1.0", "http://www.cs.umd.edu/projects/plus/SHOE/onts/cs1.0.html");
    ont = ontman.getOntology("cs-dept-ontology", "1.1", "http://www.cs.umd.edu/projects/plus/SHOE/onts/cs1.1.html");

    for (i=0; i < ids.length; i++) {
      childId = new Unique_Elt(ids[i][0], ids[i][1], ids[i][2]);
      parId = new Unique_Elt(ids[i][3], ids[i][4], ids[i][5]);
      startTime = new Date();
      answer = ontman.isSubcategoryOf(childId, parId);
      endTime = new Date();
      duration = endTime.getTime() - startTime.getTime();
      System.out.println("isSubcategoryOf(" + childId.toString() + ", " +
			 parId.toString() + " = " + answer + " (took " +
			 duration + " ms)");
    }

    /*
    enum = doc.getPSHOE();
    inst = (Instance)enum.nextElement();

    elmt_id = ontman.resolveName("cs.Student", inst);
    System.out.println("Element id is: " + elmt_id);
    */

    /*
    System.out.println("Re-formed document:");
    doc.printHTML(System.out);
    */

    /*
    System.out.println(ontman.getPrefix("cs.junk"));
    System.out.println(ontman.getPrefixRemainder("cs.junk"));
    System.out.println(ontman.getPrefix("cs.foo.junk"));
    System.out.println(ontman.getPrefixRemainder("cs.foo.junk"));
    */

    /*
      // Hashing unique elements test
    Hashtable htab;
    Unique_Elt lookId;
    String s;

    htab = new Hashtable();
    elmt_id = new Unique_Elt("cs-dept-ontology", "1.0", "Faculty");
    htab.put(elmt_id, "Found Faculty");
    elmt_id = new Unique_Elt("cs-dept-ontology", "1.0", "Student");
    htab.put(elmt_id, "Found Student");
    lookId = new Unique_Elt("cs-dept-ontology", "1.0", "Faculty");
    if (htab.containsKey(lookId))
      System.out.println("Contains key " + lookId);
    if (htab.containsKey(elmt_id))
      System.out.println("Contains key " + elmt_id);
    s = (String)htab.get(lookId);
    System.out.println("Retrieved: " + s);
    s = (String)htab.get(elmt_id);
    System.out.println("Retrieved: " + s);
    */

    /*
      // Unique element hash code test 
    hashTest("cs-dept-ontology", "1.0", "Faculty");
    hashTest("cs-dept-ontology", "1.1", "Student");
    hashTest("univ-ontology", "1.0", "Student");
    hashTest("cs-dept-ontology2", "1.0", "Student");
    hashTest("cs-dept-ontology", "1.0", "Student");
    hashTest("cs-dept-ontolog", "1.0", "Student");
    hashTest("cs-dept-ontolo", "1.0", "Student");
    hashTest("cs-dept-ontol", "1.0", "Student");
    hashTest("cs-dept-onto", "1.0", "Student");
    hashTest("general-ontology", "1.0", "Physical_Object");
    hashTest("general-ont", "1.0", "Physical_Object");
    */

  }

  public static void hashTest(String ontId, String ontVer, String name) {

    Unique_Elt elmt_id;

    elmt_id = new Unique_Elt(ontId, ontVer, name);
    System.out.print(ontId + "/" + ontVer + "/" + name + ":\t");
    System.out.println(elmt_id.hashCode());
  }

}


