/* Copyright (C) 1999 Jeff Heflin - University of Maryland
 *	
 * This software is provided under the terms of the GNU Lesser General Public
 * License Version 2.1, February 1999. See the header in SHOE_Doc.java
 * and the file License.html for details.
 */

package SHOE;

import java.lang.System;

/** A class that represents an internet wide unique identifier for
  an ontology element. */
public class Unique_Elt extends Object {

  String ont_id;
  String ont_ver;
  String name;

  /** Create a new unique element id from an ontology id, ontology
    version number and the name of an element in the ontology. */
  public Unique_Elt(String id, String ver, String nm) {

    ont_id = id;
    ont_ver = ver;
    name = nm;
  }

  /** Return the ontology id for this unique element. */
  public String getOntId() {

    return ont_id;
  }

  /** Return the ontology version number for this unique element. */
  public String getOntVer() {

    return ont_ver;
  }

  /** Return the local name of this element in its home ontology. */
  public String getName() {

    return name;
  }

  /** Return true if the other unique element is equivalent to this one. */
  public boolean equals(Object otherObj) {

    Unique_Elt otherElt;
    boolean ans = false;

    if (otherObj instanceof Unique_Elt) {
      otherElt = (Unique_Elt)otherObj;
      if (ont_id.equals(otherElt.ont_id) &&
	  ont_ver.equals(otherElt.ont_ver) &&
	  name.equals(otherElt.name))
	ans = true;
    }
    return ans;
  }

  /** Returns a hash code for the object. */
  public int hashCode() {

    // since only the first 25 characters of a String are significant, 
    // put the most unique and shortest parts first before calling the
    // string hashCode() method. */
    return (name + ont_ver + ont_id).hashCode();
  }

  /** Returns a string representation of the unique element object. */
  public String toString() {

    return ont_id + ",v." + ont_ver + ":" + name;
  }
}
