/* Copyright (C) 1999 Jeff Heflin - University of Maryland
 *	
 * This software is provided under the terms of the GNU Lesser General Public
 * License Version 2.1, February 1999. See the header in SHOE_Doc.java
 * and the file License.html for details.
 */

/* Change History:
   12/29/00, JDH: Made getUseOntNode() work with new Ontology.getUseOnts()
   11/01/00, JDH: Made the fields private.
*/

package SHOE;

import java.util.*;
import java.io.*;
import java.lang.System;
import HTML.*;
import Tree.*;

public class Use_Ontology extends SHOE_Obj {

  private String id;
  private String version;
  private String prefix;
  private String url;
  
  /** Create a new use ontology statment from an ontology id, version,
    prefix, and a URL for locating it. */
  public Use_Ontology(String id, String version, String prefix, String url)  {
	
    this.id = id;
    this.version = version;
    this.prefix = prefix;
    this.url = url;
  }


  /** Create a new use ontology statement from its SGML tag representation. */
  public Use_Ontology(SGML_Tag sgmltag) {
  
    id = sgmltag.getAttribute("ID");
    version = sgmltag.getAttribute("VERSION");
    prefix = sgmltag.getAttribute("PREFIX");
    url = sgmltag.getAttribute("URL");

  }

  /** Returns an identifier that distinguishes this Use_Ontology statement
    from all others that appear in the current Ontology or Instance. */
  public String getKey() {
  
    return prefix;
  }

  /** Returns the Id of the used ontology. */
  public String getId() {

    return id;
  }

  /** Returns the version number of the used ontology. */
  public String getVersion() {

    return version;
  }

  /** Returns the prefix that is prepended to the names of elements
    from the used ontology. */
  public String getPrefix() {

    return prefix;
  }

  /** Returns the recommended URL for locating the used ontology. */
  public String getUrl() {

    return url;
  }

  /** Returns a label suitable for identifying the Use-Ontology to
    a user. */
  public String getOntLabel() {

    return prefix +":  "+id + ", v." + version;
  }

  /** Creates a tree node for the Use-Ontology in which the children
    of the node are nodes containing the Use-Ontologies which are used
    by its ontology. This method is to be use in creating tree
    lists from which a Use-Ontology can be selected. */
  public LabeledTreeNode getUseOntNode(OntManager ontman) {

    String           label;
    LabeledTreeNode  newNode;
    Ontology         prefixOnt;              // referenced ontology
    Use_Ontology[]   useOnts;
    Use_Ontology     childUont;
    LabeledTreeNode  childNode;

    label = getOntLabel();
    newNode = new LabeledTreeNode(this, label);
    prefixOnt = ontman.getOntology(this);
    if(prefixOnt != null){
      useOnts = prefixOnt.getUseOnts();
      for (int i=0; i < useOnts.length; i++) {
	childUont = useOnts[i];
	childNode = childUont.getUseOntNode(ontman);
	newNode.add(childNode);
      }
    }
    return newNode;
  }

  /** Change the fields of the use ontology statement. */
  public void update(String id, String version, String prefix, String url)  {
	
    this.id = id;
    this.version = version;
    this.prefix = prefix;
    this.url = url;
  }

  public Vector readSubTags(HTML_Tokenizer intokens) {
    /* Use_Ontology has no sub-tags but we must still override the abstract
       class with an empty method */
    Vector errStrings = null;

    errStrings = new Vector();
    return errStrings;
  }
  
  public void printSGML(PrintStream out) {
  
    out.print(this);
  }
  
  /** Returns the string representation of the tag. Any contents
   or end tags are not included. */
  public String toString() {
  
    String str;
    
    str = "<USE-ONTOLOGY ID=\"" + id + "\"";
    str = str + " VERSION=\"" + version + "\"";
    str = str + " PREFIX=\"" + prefix + "\"";
    if (url != null)
      str = str + " URL= \"" + url + "\"";
    str = str + ">";
    return str;
  }
}

