/* Copyright (C) 1999 Jeff Heflin - University of Maryland
 *	
 * This software is provided under the terms of the GNU Lesser General Public
 * License Version 2.1, February 1999. See the header in SHOE_Doc.java
 * and the file License.html for details.
 */

package SHOE;

import java.io.*;
import java.net.*;
import java.util.Enumeration;

/** This program reads a SHOE document from a specified URL, validates
  its and outputs a list of any errors found in the document. Both
  instances and ontologies are validated. */
public class Validator {

  static OntManager ontman;

  public static void main(String args[]) throws IOException {
    
    String urlStr;
    URL    url;

    if (args.length != 1) {
      System.out.println("Usage: java SHOE.Validator url");
      System.exit(1);
    }
    else {
      urlStr = args[0];

      ontman = new OntManager();
      try {
	url = new URL(urlStr);   
	validatePage(url);
      } catch (MalformedURLException ex) {
	System.out.println("ERROR: Invalid URL");
	/* we use an exit code of 0 since this isn't a serious error */
	System.exit(0);
      }
      System.exit(0);
    }
  }


  /** Retrieve a page and store any SHOE information in the KB. */
  private static void validatePage(URL url) {

    SHOE_Doc doc;
    InputStream inStream;

    doc = new SHOE_Doc();
    try {
      inStream = url.openStream();
      doc.parse(inStream);
      inStream.close();

      doc.validate(ontman);

      if (doc.getVersion() != 1.0) {
	System.out.println("Document does not contain SHOE");
      }
      else {
	if (doc.hasParseErrors()) {
	  System.out.println("Document contains errors:");
	  System.out.println(doc.getParseErrors());
	}
	else {
	  System.out.println("Document is valid SHOE");	
	}
	System.out.println();
	printSummary(doc);
      }
    } catch (IOException ex) {
      System.out.println("ERROR: Could not read from URL");
      return;
    }
  }

  private static void printSummary(SHOE_Doc doc) {

    Enumeration shoeContents;
    SHOE_Obj    obj;
    int         ontCnt        = 0;
    int         instCnt       = 0;

    shoeContents = doc.getPSHOE();
    while (shoeContents.hasMoreElements()) {
      obj = (SHOE_Obj)shoeContents.nextElement();
      if (obj instanceof Ontology)
	ontCnt++;
      else if (obj instanceof Instance)
	instCnt++;
    }
	
    System.out.println("Contents:");
    if (ontCnt > 0)
      System.out.println("   Ontologies: " + ontCnt);
    if (instCnt > 0)
      System.out.println("   Instances:  " + instCnt);
    if (ontCnt == 0 && instCnt == 0)
      System.out.println("   No ontologies or instances");
  }
}
