package Tree;

import java.util.*;

/** This class provides an updatable tree node. In order to be compatible
  with the largest possible number of browsers we can't use Swing, since
  it is Java 1.1+. However, to ease any future upgrading, we have created
  this class to mirror that of a Swing class. It's methods are a subset
  of those in the Swing class. */
public class DefaultMutableTreeNode {

  protected Vector children;
  protected DefaultMutableTreeNode parent;
  protected Object userObject;

  public DefaultMutableTreeNode() {

    this.children = new Vector();
    this.parent = null;
    this.userObject = null;
  }

  public DefaultMutableTreeNode(Object userObject) {

    this.children = new Vector();
    this.parent = null;
    this.userObject = userObject;
  }

  /** Removes newChild from its parent and makes it a child of this node
    by adding it to the end of this node's child array. */
  public void add(DefaultMutableTreeNode newChild) {

    DefaultMutableTreeNode oldParent;

    oldParent = newChild.getParent();
    if (oldParent != null) {
      oldParent.remove(newChild);
    }
    newChild.setParent(this);
    children.addElement(newChild);
  }
    
  public Enumeration children() {

    return children.elements();
  }

  public DefaultMutableTreeNode getParent() {

    return parent;
  }

  /** Removes newChild from its present parent (if it has a parent), sets
    the child's parent to this node, and then adds the child to this
    node's child array at index childIndex. */
  public void insert(DefaultMutableTreeNode newChild, int childIndex) {

    DefaultMutableTreeNode oldParent;

    oldParent = newChild.getParent();
    if (oldParent != null) {
      oldParent.remove(newChild);
    }
    newChild.setParent(this);
    children.insertElementAt(newChild, childIndex);
  }

  public void remove(int childIndex) {

    children.removeElementAt(childIndex);
  }

  public void remove(DefaultMutableTreeNode aChild) {

    children.removeElement(aChild);
  }
 
  /** Sets this node's parent to newParent but does not change the
    parent's child array. */
  public void setParent(DefaultMutableTreeNode newParent) {

    parent = newParent;
  }

  public Object getUserObject() {

    return userObject;
  }

  public void setUserObject(Object userObject) {

    this.userObject = userObject;
  }

  /** Creates and returns an enumeration that traverses the subtree
    rooted at this node in preorder. */
  public Enumeration preorderEnumeration() {

    System.out.println("Not yet implemented!");
    return null;
  }
}
