package Tree;

/** This class provides an updatable tree node which contains an extra
  field for a user-defined label. This label can then be used as the
  string which is displayed in visual tree components. */
public class LabeledTreeNode extends DefaultMutableTreeNode {

  protected String userLabel;

  public LabeledTreeNode(String userLabel) {

    super();
    this.userLabel = userLabel;
  }

  public LabeledTreeNode(Object userObject, String userLabel) {

    super(userObject);
    this.userLabel = userLabel;
  }

  public void setLabel(String userLabel) {

    this.userLabel = userLabel;
  }

  public String getLabel() {

    return userLabel;
  }

  /** Adds a new child to the node, placing it such that the children
    are sorted by the label. Note, all children must be added using this
    routine for the entire set of children to be sored. */
  public void insertInSortedOrder(LabeledTreeNode newChild) {

    int size;
    int i;
    boolean done;
    LabeledTreeNode indxNode;                 // node indexed by i

    size = children.size();
    i = 0;
    done = false;
    while (i < size && done == false) {
      indxNode = (LabeledTreeNode)children.elementAt(i);
      if (newChild.compareTo(indxNode) < 0) {
	children.insertElementAt(newChild, i);
	done = true;
      }
      i++;
    }
    if (done == false)
      children.addElement(newChild);
  }

  /** Compares the label of this node to the node passed as an argument
    and returns a number less than 0, equal to 0, or greater than 0
    if lexiographically this node is before, equivalent to, or after
    the argument, respectively. */
  public int compareTo(LabeledTreeNode anotherNode) {

    return userLabel.compareTo(anotherNode.userLabel);
  }

}  

